<?php

namespace Drupal\okta_user_sync\Form;

use Drupal;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\okta_user_sync\Helper\MoOktaHelper;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;


class MoOktaToDrupal extends FormBase
{
  private $base_url;
  private ImmutableConfig $config;
  private Config $config_factory;
  protected $messenger;

  public function __construct(){
    global $base_url;
    $this->base_url = $base_url;
    $this->config = Drupal::config('okta_user_sync.settings');
    $this->config_factory = Drupal::configFactory()->getEditable('okta_user_sync.settings');
    $this->messenger = Drupal::messenger();
  }

  public function getFormId()
  {
    return 'MoOktaToDrupal';
  }

  public function buildForm(array $form, FormStateInterface $form_state)
  {
    $form['mo_okta_sync_add_css']=array(
      '#attached' => array(
        'library' => array(
          'okta_user_sync/okta_user_sync.admin',
          'okta_user_sync/okta_user_sync.okta_test',
          'okta_user_sync/okta_user_sync.add_trial_popup',
        )
      ),
    );

    $form['mo_summary_okta_header_style'] = [
      '#markup' =>  t('<div class="mo_okta_header_container_summary">'),
    ];

    \Drupal::configFactory()->getEditable('okta_user_sync.settings')->set('okta_user_sync_status1','okta_to_drupal')->save();

    $this->oktaManualDrupalProvisioning($form,$form_state);
    $this->schedulerBasedProvisioning($form,$form_state);
    $this->mapping($form, $form_state);
    $form['okta_to_drupal_save_button'] = [
      '#type' => 'submit',
      '#value' => t('Save Settings'),
      '#button_type'=> 'primary',
      '#disabled'=>true,
    ];
    $okta_handler = new MoOktaHelper();
    $okta_handler->moOktaShowCustomerSupportIcon($form, $form_state);
    return $form;
  }

  private function mapping(array &$form, FormStateInterface $form_state)
  {
    $mo_premium_tag ='<a href = "okta_upgrade_plans" >[PREMIUM]</a>';
    $form['mo_okta_mapping_fieldset']= [
      '#type' => 'fieldset',
      '#title' => $this->t('Mapping ').$mo_premium_tag.'<hr>',
    ];
    $this->basicAttributeMapping($form, $form_state);
    $this->customAttributeMapping($form, $form_state);
    return $form;
  }

  private function basicAttributeMapping(array &$form, FormStateInterface $form_state)
  {
    $usr = User::load(\Drupal::currentUser()->id());
    $usrVal = $usr->toArray();
    $custom_fields = [];

    foreach ($usrVal as $key => $value) {
      $custom_fields[$key] = $key;
    }

    $form['mo_okta_mapping_fieldset']['okta_attribute_mapping']=[
      '#type' => 'fieldset',
      '#attributes' => ['style' => 'box-shadow: none'],
      '#title' => $this->t('Basic Attribute Mapping<hr>'),
      '#disabled'=>true,

    ];
    $username_attr = $this->config->get('okta_user_sync_basic_username_mapping');
    $username_attr = $username_attr ?? 'mail';
    $mail_attr = $this->config->get('okta_user_sync_basic_mail_mapping');
    $mail_attr = $mail_attr ?? 'mail';

    $form['mo_okta_mapping_fieldset']['okta_attribute_mapping']['okta_username_attribute'] =[
      '#type' => 'select',
      '#title' => t('Username Attribute'),
      '#attributes' => array('style' => 'width:75%','placeholder' => 'Enter Username attribute'),
      '#default_value' =>$custom_fields[$username_attr],
      '#options'=>$custom_fields,
      '#prefix' => '<div><table class="mo_basic_attribute_mapping_table"><tr><td>',
      '#suffix' => '</td>',
    ];


    $form['mo_okta_mapping_fieldset']['okta_attribute_mapping']['okta_email_attribute'] = [
      '#type' => 'select',
      '#title' => t('Email Attribute'),
      '#default_value' =>$custom_fields[$mail_attr],
      '#options'=> $custom_fields,
      '#attributes' => array('style' => 'width:75%','placeholder' => 'Enter Email attribute'),
      '#prefix'=>'<td>',
      '#suffix'=>'</td></tr></table></div>',
    ];

    return $form;
  }

  private function customAttributeMapping(array &$form, FormStateInterface $form_state)
  {


    $usr = User::load(\Drupal::currentUser()->id());
    $usrVal = $usr->toArray();
    $custom_fields = [];

    foreach ($usrVal as $key => $value) {
      $custom_fields[$key] = $key;
    }

    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping'] =[
      '#type' => 'fieldset',
      '#title' => t('Custom Attribute Mapping').'<hr>',
      '#attributes' => ['style' => 'box-shadow: none'],
    ];

    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping']['attribute_mapping_info'] = [
      '#markup' => '<div class="mo_okta_highlight_background">This feature allows you to map the user attributes from your Drupal to Okta as well as from Okta to Drupal.</div>',
    ];



    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping']['drupal_attr_name'] = [
      '#prefix' => '<div><table class="mo_custom_attribute_mapping_table"><tr><td>',
      '#title' => t('Drupal Attribute Name'),
      '#type' => 'select',
      '#options'=>$custom_fields,
      '#disabled' => TRUE,
      '#attributes' => array('style' => 'width:100% !important;'),
      '#suffix'=>'</td>',
    ];

    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping']['okta_attribute_name'] = [
      '#type' => 'textfield',
      '#id' => 'text_field1',
      '#prefix' => '<td>',
      '#suffix' => '</td>',
      '#title' => t('Okta Attribute Name'),
      '#attributes' => array('style' => 'width:100%','placeholder' => 'Enter Okta Attribute Name'),
      '#required' => FALSE,
      '#disabled' => TRUE,
    ];

    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping']['okta_add_name'] = [
      '#prefix' => '<td>',
      '#suffix' => '</td>',
      '#type' => 'button',
      '#disabled' => 'true',
      '#attributes' => array('style' => 'float: right;'),
      '#value' => '+',
    ];

    $form['mo_okta_mapping_fieldset']['mo_okta_automatic_prov_custom_attribute_mapping']['okta_sub_name'] = [
      '#prefix' => '<td>',
      '#suffix' => '</td></tr></table></div>',
      '#type' => 'button',
      '#disabled' => 'true',
      '#value' => '-',
    ];
    return $form;
  }

  public function saveSummaryConfiguration(array $form, FormStateInterface $form_state)
  {
    $form_values = ($form_state->getValues())['mo_summary_okta_configuration_table'];
    $data=[
      'Okta Portal URL:' => 'okta_user_sync_base_url',
      'Okta Bearer Token:' => 'okta_user_sync_bearer_token',
      'Test Your Configuration<br>Enter user email:'=>'okta_user_sync_upn',
    ];
    foreach ($data as $key => $value){
      $this->config_factory->set($value, $form_values[$key][$value])->save();
    }
    $bearer_token = $this->config_factory->get('okta_user_sync_bearer_token');
    $provider_base_url= $this->config_factory->get('okta_user_sync_base_url');
    Drupal::configFactory()->getEditable('user_provisioning.settings')->set('mo_user_provisioning_configured_application', 'okta')
      ->set('mo_provider_specific_provisioning_base_url', $provider_base_url)
      ->set('mo_provider_specific_provisioning_api_token',$bearer_token)
      ->save();

    $okta_helper = new MoOktaHelper();
    $okta_helper->fetchAttributes();

    $okta_test_data = Json::decode($this->config->get('mo_okta_attr_list_from_server'));
    if($okta_test_data != '')
    {
      if (isset($okta_test_data["errorCode"])){
        $this->messenger->addError(t('An error occurred while performing test configuration. Please refer to <a href="#test_config_result">Test Configuration Result</a> for more information.'));
      }else{
        $this->messenger->addstatus(t('Test Configuration successful. You can check the complete list of attributes received from Okta by clicking on the link <a href="#test_config_result">HERE</a>.'));
      }
    }
    return $form;
  }

  public function backFunMoOktaToDrupal(array $form, FormStateInterface $form_state)
  {
    $status = $this->config->get('okta_user_sync_status');
    if($status ==  'overview'){
      $response = new RedirectResponse($this->base_url.'/admin/config/people/okta_user_sync/overview');
    }else{
      $response = new RedirectResponse($this->base_url.'/admin/config/people/okta_user_sync/overview?tab='.$status);
    }
    $response->send();
    return new Response();

  }

  private function oktaManualDrupalProvisioning(array &$form, FormStateInterface $form_state){
    $mo_premium_tag ='<a href = "okta_upgrade_plans" >[PREMIUM]</a>';

    $form['mo_okta_provisioning_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => t('Manual Provisioning Configuration ').$mo_premium_tag.'<hr>',
      '#attributes' => ['style' => 'box-shadow: none,', 'class'=>['field_set_label_css']],
    ];

    $form['mo_okta_provisioning_fieldset']['okta_provisioning_operations'] = [
      '#type' => 'table',
      '#responsive' => true,
      '#attributes' => ['class' => ['mo_okta_provisioning_operations_table']],

    ];

    $row = $this->moProvisioningOperations();
    $form['mo_okta_provisioning_fieldset']['okta_provisioning_operations']['operations'] = $row;

    $form['mo_okta_provisioning_fieldset']['create_user_fieldset']=[
      '#type' => 'fieldset',
      '#title' => $this->t('Sync Users ').'<hr>',
      '#attributes' => ['style' => 'box-shadow: none'],
    ];

    $form['mo_okta_provisioning_fieldset']['create_user_fieldset']['mo_okta_drupal_username'] = [
      '#type' => 'textfield',
      '#attributes' => ['placeholder' => 'Search Okta username of user to sync'],
      '#autocomplete_route_name' => 'user_provisioning.autocomplete',
      '#disabled' => true,
      '#prefix' => '<p class="mo_okta_highlight_background"><strong>Note:</strong> Search the username of user present in Okta to sync it to the Drupal.</p><div class="container-inline">',
    ];

    $form['mo_okta_provisioning_fieldset']['create_user_fieldset']['mo_okta_sync_button'] = [
      '#type' => 'submit',
      '#value' => t('Sync'),
      '#button_type' => 'primary',
      '#attributes' => ['class' => ['mo_okta_sync_button']],
      '#disabled' => true,
    ];

    $form['mo_okta_provisioning_fieldset']['create_user_fieldset']['mo_okta_sync_all_button'] = [
      '#type' => 'submit',
      '#value' => t('Sync All Users'),
      '#button_type' => 'primary',
      '#attributes' => ['class' => ['mo_okta_sync_all_button']],
      '#disabled' => true,
    ];

    return $form;

  }
  private function schedulerBasedProvisioning(array &$form, FormStateInterface $form_state){
    $mo_premium_tag ='<a href = "okta_upgrade_plans" >[PREMIUM]</a>';

    $form['mo_okta_scheduler_provisioning_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => t('Scheduler Based Provisioning Configuration ').$mo_premium_tag.'<hr>',
      '#attributes' => ['class'=>['field_set_label_css']],
    ];

    $form['mo_okta_scheduler_provisioning_fieldset'] ['okta_provisioning_operations'] = [
      '#type' => 'table',
      '#responsive' => true,
      '#attributes' => ['class' => ['mo_okta_provisioning_operations_table']],

    ];

    $row = $this->moProvisioningOperations();
    $form['mo_okta_scheduler_provisioning_fieldset'] ['okta_provisioning_operations']['operations'] = $row;


    $form['mo_okta_scheduler_provisioning_fieldset'] ['mo_okta_scheduler_provisioning_configuration']= [
      '#type' => 'fieldset',
      '#title' => $this->t('Configuration ').'<hr>',
      '#attributes' => ['style' => 'box-shadow: none'],
    ];

//    $form['mo_okta_scheduler_provisioning_fieldset']['enable_auto_creation'] = [
//      '#type' => 'checkbox',
//      '#title' => t('Enable Scheduler-based Automatic User Creation'),
//      '#disabled' => true,
//    ];

    $form['mo_okta_scheduler_provisioning_fieldset']['mo_okta_scheduler_provisioning_configuration']['auto_creation'] = [
      '#type' => 'number',
      '#title' => t('Fetch All users details in next * minutes'),
      '#attributes'=>array('style' => 'width: 45%',),
      '#disabled' => true,

    ];

//    $form['mo_okta_scheduler_provisioning_fieldset']['mo_okta_scheduler_provisioning_configuration']['save_button'] = [
//      '#type' => 'submit',
//      '#value' => t('Save Configuration'),
//      '#disabled' => true,
//    ];
    return $form;
  }

  private function moProvisioningOperations(): array
  {
    $row['read_user_okta'] = [
      '#type' => 'checkbox',
      '#title' => t('Read user'),
      '#disabled' => true,
      '#prefix'=> '<div class="container-inline">',
    ];

    $row['create_user_okta'] = [
      '#type' => 'checkbox',
      '#title' => t('Create user '),
      '#disabled' => true,
    ];

    $row['update_user_okta'] = [
      '#type' => 'checkbox',
      '#title' => t('Update user'),
      '#disabled' => true,
    ];

    $row['deactivate_user_okta'] = [
      '#type' => 'checkbox',
      '#title' => t('Deactivate user '),
      '#disabled' => true,
    ];

    $row['delete_user_okta'] = [
      '#type' => 'checkbox',
      '#title' => t('Delete user'),
      '#disabled' => true,
    ];

//    $row['manual_provisioning_save_button'] = [
//      '#type' => 'submit',
//      '#value' => t('Save'),
//      '#button_type' => 'primary',
//      '#disabled' => true,
//      '#suffix' => '</div>',
//    ];
    return $row;
  }

  private function attributeMapping(array &$form, FormStateInterface $form_state){

    $mo_premium_tag ='<a href = "okta_upgrade_plans" >[PREMIUM]</a>';

    $form['mo_okta_attribute_mapping_details'] =[
      '#type' => 'details',
      '#open' => false,
      '#title' => t('Mapping'),
    ];

    $form['mo_okta_attribute_mapping_details']['okta_custom_attribute_mapping'] = array(
      '#type' => 'fieldset',
      '#title' => t('Custom Attribute Mapping ').$mo_premium_tag.'<hr>',
    );

    $form['mo_okta_attribute_mapping_details']['okta_custom_attribute_mapping']['attribute_mapping_info'] = array(
      '#markup' => '<div class="mo_okta_highlight_background">This feature allows you to map the user attributes from your Drupal to Okta.</div>',
    );

    $custom_fields = [];
    $usr = User::load(\Drupal::currentUser()->id());
    $usrVal = $usr->toArray();
    foreach ($usrVal as $key => $value) {
      $custom_fields[$key] = $key;
    }

    $form['mo_okta_attribute_mapping_details']['okta_custom_attribute_mapping']['okta_custom_attribute_mapping_table'] = [
      '#type' => 'table',
      '#responsive' => TRUE ,
      '#attributes' => ['class' => ['custom_attribute_mapping_table']]
    ];

    $row = $this->moOktaCustomAttributeMapTable($custom_fields);
    $form['mo_okta_attribute_mapping_details']['okta_custom_attribute_mapping']['okta_custom_attribute_mapping_table']['custom_mapping'] = $row;
  }

  private function moOktaCustomAttributeMapTable($custom_fields): array
  {
    $row['okta_drupal_attr_name'] = [
      '#title' => t('Drupal Attribute Name'),
      '#type' => 'select',
      '#options' => $custom_fields,
      '#disabled' => true,
    ];

    $row['okta_attribute_name'] = [
      '#type' => 'textfield',
      '#title' => t('Okta Attribute Name'),
      '#disabled' => true,
    ];

    $row['okta_add_button'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#disabled' => true,
      '#value' => '+',
    ];

    $row['okta_sub_button'] = [
      '#type' => 'submit',
      '#button_type' => 'danger',
      '#disabled' => true,
      '#value' => '-',
    ];

    return $row;
  }

  private function moOktaAttributeMapTable(): array
  {
    $row['username_attribute'] = [
      '#type' => 'select',
      '#title' => t('Username Attribute'),
      '#options' => [ 1 => 'name'],
      '#disabled' => true,
    ];

    $row['email_attribute'] = [
      '#type' => 'select',
      '#title' => t('Email Attribute'),
      '#options' => [ 1 => 'mail'],
      '#disabled' => true,
    ];

    return $row;
  }

  public function submitForm(array &$form, FormStateInterface $form_state)
  {
    // TODO: Implement submitForm() method.
  }
}
