<?php

namespace Drupal\okta_user_sync;

use Drupal;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\user_provisioning\moUserProvisioningConstants;
use Drupal\user_provisioning\moUserProvisioningCustomer;
use Drupal\user_provisioning\moUserProvisioningSupport;
use Drupal\user_provisioning\moUserProvisioningUtilities;



class MoOktaSupport
{
  private ImmutableConfig $config;
  public string $email;
  public string $query;
  public string $query_description;
  public string $query_type;
  public function __construct($email, $query, $query_description, $query_type){
    $this->email = $email;
    $this->query = $query;
    $this->query_description = $query_description;
    $this->query_type = $query_type;

    $this->config = Drupal::config('user_provisioning.settings');
  }

  public function sendSupportQuery(): bool
  {
    $modules_info = Drupal::service('extension.list.module')->getExtensionInfo('okta_user_sync');

    $modules_version = $modules_info['version'];
    $customerKey = $this->config->get('mo_user_provisioning_customer_id');
    $apikey = $this->config->get('mo_user_provisioning_customer_api_key');

    if ($customerKey == '') {
      $customerKey = "16555";
      $apikey = "fFd2XcvTGDemZvbw1bcUesNJWEqKbbUq";
    }

    $currentTimeInMillis = moUserProvisioningSupport::getTimestamp();
    $stringToHash = $customerKey . $currentTimeInMillis . $apikey;
    $hashValue = hash("sha512", $stringToHash);

    $header = array('Content-Type'=> 'application/json',
      'Customer-Key' => $customerKey,
      'Timestamp' => $currentTimeInMillis,
      'Authorization' => $hashValue);

    if ($this->query_type == 'Trial Request' || $this->query_type == 'Contact Support') {

      $url = moUserProvisioningConstants::MO_NOTIFY_SEND;
      $request_for = $this->query_type == 'Trial Request' ? 'Trial' : 'Support';
      $subject = $request_for.' request for Drupal-' . \DRUPAL::VERSION . ' Okta User Sync Module | ' .$modules_version;
      $this->query_description = ($this->query_type == 'Contact Support' ? 'Query - ' : $request_for.' requested for - ') . $this->query_description;

      $query = $this->query_type == 'Contact Support' ? '<strong>Support needed for:</strong> ' : 'Query: ';
      $content = '<div >Hello, <br><br>Company :<a href="' . $_SERVER['SERVER_NAME'] . '" target="_blank" >' . $_SERVER['SERVER_NAME'] . '</a><br><br>'. $query.$this->query . '<br><br>Email:<a href="mailto:' . $this->email . '" target="_blank">' . $this->email . '</a><br><br>Query:[DRUPAL ' . moUserProvisioningUtilities::mo_get_drupal_core_version() . ' Okta User Sync Free | PHP ' . phpversion() . ' | ' . $modules_version . ' ] ' . $this->query_description . '</div>';

      $fields = array(
        'customerKey' => $customerKey,
        'sendEmail' => true,
        'email' => array(
          'customerKey' => $customerKey,
          'fromEmail' => $this->email,
          'fromName' => 'miniOrange',
          'toEmail' => moUserProvisioningConstants::SUPPORT_EMAIL,
          'toName' => moUserProvisioningConstants::SUPPORT_EMAIL,
          'subject' => $subject,
          'content' => $content
        ),
      );



    } else {
      $url = moUserProvisioningConstants::MO_NOTIFY_SEND;

      $subject = 'Query request for Drupal-' . \DRUPAL::VERSION . ' Okta User Sync (Free) | ' . $modules_version;

      $query1 = 'Phone Number:';
      $content = '<div >Hello, <br><br>Company :<a href="' . $_SERVER['SERVER_NAME'] . '" target="_blank" >' . $_SERVER['SERVER_NAME'] . '</a><br><br>'. $query1.$this->query . '<br><br>Email:<a href="mailto:' . $this->email . '" target="_blank">' . $this->email . '</a><br><br>Query:[DRUPAL ' . moUserProvisioningUtilities::mo_get_drupal_core_version() . ' Okta User Sync Free | PHP ' . phpversion() . ' | ' . $modules_version . ' ] ' . $this->query_description . '</div>';

      $fields = array(
        'customerKey' => $customerKey,
        'sendEmail' => true,
        'email' => array(
          'customerKey' => $customerKey,
          'fromEmail' => $this->email,
          'fromName' => 'miniOrange',
          'toEmail' => moUserProvisioningConstants::SUPPORT_EMAIL,
          'toName' => moUserProvisioningConstants::SUPPORT_EMAIL,
          'subject' => $subject,
          'content' => $content
        ),
      );

    }

    $field_string = json_encode($fields);
    $mo_user_provisioning_customer = new moUserProvisioningCustomer(null, null, null, null);
    $response = $mo_user_provisioning_customer->callService( $url, $field_string, $header);

    return TRUE;
  }


}
